@@ -16,7 +16,7 @@ from strawberry_django_plus.field import connection, field, node
 from strawberry_django_plus.filters import filter
 from strawberry_django_plus.mutations.fields import create as create_mutation
 from strawberry_django_plus.mutations.fields import delete as delete_mutation
-from strawberry_django_plus.mutations.fields import input_mutation
+from strawberry_django_plus.mutations.fields import input_mutation, mutation
 from strawberry_django_plus.mutations.fields import update as update_mutation
 from strawberry_django_plus.ordering import order
 from strawberry_django_plus.type import input, interface, partial, type
@@ -41,6 +41,7 @@ __all__ = [
     "node",
     "connection",
     "input",
+    "mutation",
     "input_mutation",
     "interface",
     "mutations",
