@@ -35,6 +35,9 @@ from strawberry_django.type import StrawberryDjangoType as _StraberryDjangoType
 from strawberry_django.utils import get_annotations, is_similar_django_type
 from typing_extensions import Annotated
 
+from strawberry_django_plus.optimizer import OptimizerStore, PrefetchType
+from strawberry_django_plus.utils.typing import TypeOrSequence
+
 from . import field
 from .descriptors import ModelProperty
 from .field import StrawberryDjangoField, connection, node
@@ -83,11 +86,9 @@ def _from_django_type(
     else:
         is_connection = False
 
-    if isinstance(attr, StrawberryDjangoField) and not attr.origin_django_type:
+    if is_connection or isinstance(attr, ConnectionField):
         field = attr
-    elif is_connection or isinstance(attr, ConnectionField):
-        field = attr
-        if not isinstance(field, Connection):
+        if not isinstance(field, ConnectionField):
             field = connection()
 
         field = cast(StrawberryDjangoField, field)
@@ -101,6 +102,8 @@ def _from_django_type(
             field.base_resolver = StrawberryResolver(conn_resolver)
             if type_annotation is not None:
                 field.type_annotation = type_annotation
+    elif isinstance(attr, StrawberryDjangoField) and not attr.origin_django_type:
+        field = attr
     elif isinstance(attr, dataclasses.Field):
         default = getattr(attr, "default", UNSET)
         if default is dataclasses.MISSING:
@@ -253,6 +256,10 @@ def _process_type(
     filters: Optional[type] = UNSET,
     order: Optional[type] = UNSET,
     pagination: Optional[bool] = UNSET,
+    only: Optional[TypeOrSequence[str]] = None,
+    select_related: Optional[TypeOrSequence[str]] = None,
+    prefetch_related: Optional[TypeOrSequence[PrefetchType]] = None,
+    disable_optimization: bool = False,
     **kwargs,
 ) -> _O:
     original_annotations = cls.__dict__.get("__annotations__", {})
@@ -266,6 +273,12 @@ def _process_type(
         filters=filters,
         order=order,
         pagination=pagination,
+        disable_optimization=disable_optimization,
+        store=OptimizerStore.with_hints(
+            only=only,
+            select_related=select_related,
+            prefetch_related=prefetch_related,
+        ),
     )
 
     fields = list(_get_fields(django_type).values())
@@ -281,10 +294,8 @@ def _process_type(
         setattr(cls, f.name, f)
 
     # Make sure model is also considered a "virtual subclass" of cls
-    is_type_of = [lambda obj, info: isinstance(obj, (cls, model))]
-    if hasattr(cls, "is_type_of"):
-        is_type_of.append(cls.is_type_of)  # type:ignore
-    cls.is_type_of = lambda obj, info: any(f(obj, info) for f in is_type_of)  # type:ignore
+    if "is_type_of" not in cls.__dict__:
+        cls.is_type_of = lambda obj, info: isinstance(obj, (cls, model))  # type:ignore
 
     # Default querying methods for relay
     if issubclass(cls, Node):
@@ -301,7 +312,7 @@ def _process_type(
                 cls,
             )
 
-        if not _has_own_node_resolver(cls, "resolve_connection_resolver"):
+        if not _has_own_node_resolver(cls, "resolve_connection"):
             cls.resolve_connection = types.MethodType(
                 lambda *args, **kwargs: resolve_connection(
                     *args,
@@ -338,6 +349,8 @@ class StrawberryDjangoType(Generic[_O, _M], _StraberryDjangoType):
     order: Optional[Union[type, UnsetType]]
     filters: Optional[Union[type, UnsetType]]
     pagination: Optional[Union[bool, UnsetType]]
+    disable_optimization: bool
+    store: OptimizerStore
 
 
 @__dataclass_transform__(
@@ -365,6 +378,10 @@ def type(  # noqa:A001
     filters: Optional[type] = UNSET,
     pagination: Optional[bool] = UNSET,
     order: Optional[type] = UNSET,
+    only: Optional[TypeOrSequence[str]] = None,
+    select_related: Optional[TypeOrSequence[str]] = None,
+    prefetch_related: Optional[TypeOrSequence[PrefetchType]] = None,
+    disable_optimization: bool = False,
 ) -> Callable[[_T], _T]:
     """Annotates a class as a Django GraphQL type.
 
@@ -392,6 +409,10 @@ def type(  # noqa:A001
             filters=filters,
             pagination=pagination,
             order=order,
+            only=only,
+            select_related=select_related,
+            prefetch_related=prefetch_related,
+            disable_optimization=disable_optimization,
         )
 
     return wrapper
